"""
test nastranio.mesh_api.mod.py

Read a gmsh mesh file

(Pdb++) act_coords
     csys     X     Y     Z
gid
1       0   0.0   0.0   0.0
2       0 -50.0   0.0   0.0
3       0 -50.0  30.0   0.0
4       0   0.0  30.0   0.0
5       0   0.0   0.0  10.0
6       0 -50.0   0.0  10.0
7       0 -50.0  30.0  10.0
8       0   0.0  30.0  10.0
"""
from io import StringIO
from pathlib import Path

import numpy as np
import pandas as pd
import pytest

from nastranio.registry import Registry


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = Path(request.module.__file__)
    return filename.parent / "test_mod"


@pytest.fixture
def filled_reg(datadir):
    bulkpath = datadir / "frame_1d.nas"
    _reg = Registry()
    _reg.from_bulkfile(bulkpath, nbprocs=8, progress=False)
    _reg.bind_mod_api(index_elems=("1d",))
    return _reg


def test_initial_state(filled_reg):
    reg = filled_reg
    grids = reg.container["bulk"]["GRID"]
    act_coords = grids.coords(asdf=True)
    expected_coords = {
        "csys": {1: 0, 2: 0, 3: 0, 4: 0, 5: 0, 6: 0, 7: 0, 8: 0},
        "X": {1: 0.0, 2: -50.0, 3: -50.0, 4: 0.0, 5: 0.0, 6: -50.0, 7: -50.0, 8: 0.0},
        "Y": {1: 0.0, 2: 0.0, 3: 30.0, 4: 30.0, 5: 0.0, 6: 0.0, 7: 30.0, 8: 30.0},
        "Z": {1: 0.0, 2: 0.0, 3: 0.0, 4: 0.0, 5: 10.0, 6: 10.0, 7: 10.0, 8: 10.0},
    }

    expected_coords = pd.DataFrame(expected_coords)
    expected_coords.index.names = ["gid"]
    diff = act_coords - expected_coords
    assert np.alltrue(diff == 0)


def test_move_node_by(filled_reg):
    """move Node2 to [-49.6, 0.5, -0.5], using vector"""
    reg = filled_reg
    reg.mesh.mod.node_move_by(2, [0.4, 0.5, -0.5])
    grids = reg.container["bulk"]["GRID"]
    act_coords = grids.coords(asdf=True)
    expected_coords = {
        "csys": {1: 0, 2: 0, 3: 0, 4: 0, 5: 0, 6: 0, 7: 0, 8: 0},
        "X": {1: 0.0, 2: -49.6, 3: -50.0, 4: 0.0, 5: 0.0, 6: -50.0, 7: -50.0, 8: 0.0},
        "Y": {1: 0.0, 2: 0.5, 3: 30.0, 4: 30.0, 5: 0.0, 6: 0.0, 7: 30.0, 8: 30.0},
        "Z": {1: 0.0, 2: -0.5, 3: 0.0, 4: 0.0, 5: 10.0, 6: 10.0, 7: 10.0, 8: 10.0},
    }

    expected_coords = pd.DataFrame(expected_coords)
    expected_coords.index.names = ["gid"]
    diff = act_coords - expected_coords
    assert np.alltrue(diff == 0)
    modified_bulk = reg.to_nastran()
    fh = StringIO()
    fh.write(modified_bulk)
    fh.seek(0)
    reg2 = Registry()
    reg2.from_bulkfile(fh)
    checked_coords = reg2.container["bulk"]["GRID"].coords(asdf=True).to_dict()
    assert checked_coords == expected_coords.to_dict()


def test_node_move_to(filled_reg):
    """move Node2 to [-49.6, 0.5, -0.5], using vector"""
    reg = filled_reg
    reg.mesh.mod.node_move_to(2, [-49.6, 0.5, -0.5])
    grids = reg.container["bulk"]["GRID"]
    act_coords = grids.coords(asdf=True)
    expected_coords = {
        "csys": {1: 0, 2: 0, 3: 0, 4: 0, 5: 0, 6: 0, 7: 0, 8: 0},
        "X": {1: 0.0, 2: -49.6, 3: -50.0, 4: 0.0, 5: 0.0, 6: -50.0, 7: -50.0, 8: 0.0},
        "Y": {1: 0.0, 2: 0.5, 3: 30.0, 4: 30.0, 5: 0.0, 6: 0.0, 7: 30.0, 8: 30.0},
        "Z": {1: 0.0, 2: -0.5, 3: 0.0, 4: 0.0, 5: 10.0, 6: 10.0, 7: 10.0, 8: 10.0},
    }

    expected_coords = pd.DataFrame(expected_coords)
    expected_coords.index.names = ["gid"]
    diff = act_coords - expected_coords
    assert np.alltrue(diff == 0)
    modified_bulk = reg.to_nastran()
    fh = StringIO()
    fh.write(modified_bulk)
    fh.seek(0)
    reg2 = Registry()
    reg2.from_bulkfile(fh)
    checked_coords = reg2.container["bulk"]["GRID"].coords(asdf=True).to_dict()
    assert checked_coords == expected_coords.to_dict()


def test_nearest_grid(filled_reg):
    reg = filled_reg
    closest_gids = reg.mesh.mod.rtrees.nearest_grids(
        coords=(-50, 30, 0), num_results=3, astype=list
    )
    assert closest_gids == [3, 7, 2]
    # grids = reg.container["bulk"]["GRID"]
    # act_coords = grids.coords(asdf=True)
    closest_gids = reg.mesh.mod.rtrees.nearest_grids(
        coords=(-49.9, 30.2, 0.1), num_results=3, astype=list
    )
    assert closest_gids == [3, 7, 2]
    # -------------------------------------------------------------------------
    # nearest elements
    closest_eids = reg.mesh.mod.rtrees.nearest_elements(
        coords=(-49.9, 30.2, 0.1), num_results=2, astype=list, cardname=None
    )
    assert closest_eids == {"CBAR": [11, 20]}

    closest_eids = reg.mesh.mod.rtrees.nearest_elements(
        coords=(-49.9, 30.2, 0.1), num_results=2, astype=list, cardname="CBAR"
    )
    assert closest_eids == [11, 20]


def test_node_add(filled_reg):
    reg = filled_reg
    coords = reg.container["bulk"]["GRID"].coords(asdf=True)
    assert len(coords) == 8  # not modified mesh
    # -------------------------------------------------------------------------
    # check that creating a node with an exiting gid raise a ValueError
    with pytest.raises(ValueError) as e_info:
        reg.mesh.mod.node_add(gid=1, coords=(10, 11, 12))
    # -------------------------------------------------------------------------
    # creating a node
    created_gid = reg.mesh.mod.node_add(coords=(10.1, 11, 12), clear_caches=True)
    assert created_gid == 9
    coords = reg.container["bulk"]["GRID"].coords(asdf=True)
    assert len(coords) == 9
    assert coords.loc[9].to_dict() == {"csys": 0.0, "X": 10.1, "Y": 11.0, "Z": 12.0}
    # -------------------------------------------------------------------------
    # creating a node
    created_gid = reg.mesh.mod.node_add(
        gid=100, coords=(10.1, 11, -12), clear_caches=True
    )
    assert created_gid == 100
    coords = reg.container["bulk"]["GRID"].coords(asdf=True)
    assert len(coords) == 10
    assert coords.loc[100].to_dict() == {"csys": 0.0, "X": 10.1, "Y": 11.0, "Z": -12.0}
    # -------------------------------------------------------------------------
    # check that RTREES are updated
    closest_gids = reg.mesh.mod.rtrees.nearest_grids(
        coords=(10.1, 11, -12), num_results=1, astype=list
    )
    assert closest_gids == [100]


def test_split1d_knowing_eid(filled_reg):
    reg = filled_reg
    card, new_eid, new_gid = reg.mesh.mod.elem1d_split(
        eid=12, where=0.25, clear_caches=True
    )
    assert reg.mesh.eid2gids()[new_eid] == {9, 1}
    assert reg.mesh.eid2gids(keep_order=True)[new_eid] == [9, 1]
    assert reg.mesh.eid2gids()[12] == {4, 9}
    assert reg.mesh.eid2gids(keep_order=True)[12] == [4, 9]

    assert card.eid2gids()[new_eid] == {9, 1}
    assert card.eid2gids(keep_order=True)[new_eid] == [9, 1]
    assert card.eid2gids()[12] == {4, 9}
    assert card.eid2gids(keep_order=True)[12] == [4, 9]
    assert reg.mesh.coords(asdf=True).loc[new_gid][["X", "Y", "Z"]].to_dict() == {
        "X": 0.0,
        "Y": 22.5,
        "Z": 0.0,
    }
    directions = reg.mesh.normals().loc[[12, new_eid]].to_dict()
    assert directions == {
        "X": {12: 0.0, 21: 0.0},
        "Y": {12: -1.0, 21: -1.0},
        "Z": {12: 0.0, 21: 0.0},
    }


def test_multiple_split1d_all_CBAR(filled_reg):
    reg = filled_reg
    points = [
        [-12.5, 1, 11],  # on eid13
        [-25, -2, -16],  # on eid9
        [5, 3, -2],  # on eid12
        [-49.99, -1, -11],  # should be merged with node gid2
        [-37.5, -2, 16],  # on eid13
    ]
    details = reg.mesh.mod.multiple_elem1d_split(points, "CBAR")
    # reg.to_nastran("/home/nic/toto2.nas")
    exp = [
        {"XA": -12.5, "XB": 0.0, "XC": 10.0, "eid": 21, "gid": 9, "prev_eid": 13},
        {"XA": -25.0, "XB": 0.0, "XC": 0.0, "eid": 22, "gid": 10, "prev_eid": 9},
        {"XA": 0.0, "XB": 3.0, "XC": 0.0, "eid": 23, "gid": 11, "prev_eid": 12},
        {"XA": -50.0, "XB": 0.0, "XC": 0.0, "eid": None, "gid": 2, "prev_eid": None},
        {"XA": -37.5, "XB": 0.0, "XC": 10.0, "eid": 24, "gid": 12, "prev_eid": 13},
    ]
    assert details == exp
    assert reg.mesh.eid2card()[22] == "CBAR"
    assert reg.mesh.eid2gids()[22] == {2, 10}
    assert reg.mesh.eid2gids(keep_order=True)[22] == [10, 2]
    eids_bbox = reg.mesh.eid2bbox()
    eid22_bbox = eids_bbox.loc[22].to_dict()
    assert eid22_bbox == {
        "X1_min": -50.0,
        "X2_min": 0.0,
        "X3_min": 0.0,
        "X1_max": -25.0,
        "X2_max": 0.0,
        "X3_max": 0.0,
    }


def test_multiple_split1d_on_specific_rtree(filled_reg):
    reg = filled_reg
    # reg.to_nastran("/home/nic/toto.nas")
    points = [
        [-12.5, 1, 11],  # on eid13
        [-25, -2, -16],  # on eid13
        [5, 3, -2],  # on eid16
        [-49.99, -1, -11],  # should be merged with node gid6
        [-37.5, -2, 16],  # on eid13
    ]
    rtree = reg.mesh.mod.rtrees.create_rtree(eids=(13, 14, 15, 16))
    details = reg.mesh.mod.multiple_elem1d_split(points, "CBAR", rtree=rtree)
    # reg.to_nastran("/home/nic/toto3.nas")
    exp = [
        {"XA": -12.5, "XB": 0.0, "XC": 10.0, "eid": 21, "gid": 9, "prev_eid": 13},
        {"XA": -25.0, "XB": 0.0, "XC": 10.0, "eid": 22, "gid": 10, "prev_eid": 13},
        {"XA": 0.0, "XB": 3.0, "XC": 10.0, "eid": 23, "gid": 11, "prev_eid": 16},
        {"XA": -50.0, "XB": 0.0, "XC": 10.0, "eid": None, "gid": 6, "prev_eid": None},
        {"XA": -37.5, "XB": 0.0, "XC": 10.0, "eid": 24, "gid": 12, "prev_eid": 13},
    ]
    assert details == exp
    assert reg.mesh.eid2card()[22] == "CBAR"
    assert reg.mesh.eid2gids()[22] == {10, 12}
    assert reg.mesh.eid2gids(keep_order=True)[22] == [10, 12]
    eids_bbox = reg.mesh.eid2bbox()
    eid22_bbox = eids_bbox.loc[22].to_dict()
    assert eid22_bbox == {
        "X1_min": -37.5,
        "X2_min": 0.0,
        "X3_min": 10.0,
        "X1_max": -25.0,
        "X2_max": 0.0,
        "X3_max": 10.0,
    }


def test_multiple_split1d_on_specific_eids(filled_reg):
    reg = filled_reg
    # reg.to_nastran("/home/nic/toto.nas")
    points = [
        [-12.5, 1, 11],  # on eid13
        [-25, -2, -16],  # on eid13
        [5, 3, -2],  # on eid16
        [-49.99, -1, -11],  # should be merged with node gid6
        [-37.5, -2, 16],  # on eid13
    ]
    details = reg.mesh.mod.multiple_elem1d_split(points, "CBAR", eids=(13, 14, 15, 16))
    # reg.to_nastran("/home/nic/toto3.nas")
    exp = [
        {"XA": -12.5, "XB": 0.0, "XC": 10.0, "eid": 21, "gid": 9, "prev_eid": 13},
        {"XA": -25.0, "XB": 0.0, "XC": 10.0, "eid": 22, "gid": 10, "prev_eid": 13},
        {"XA": 0.0, "XB": 3.0, "XC": 10.0, "eid": 23, "gid": 11, "prev_eid": 16},
        {"XA": -50.0, "XB": 0.0, "XC": 10.0, "eid": None, "gid": 6, "prev_eid": None},
        {"XA": -37.5, "XB": 0.0, "XC": 10.0, "eid": 24, "gid": 12, "prev_eid": 13},
    ]
    assert details == exp
    assert reg.mesh.eid2card()[22] == "CBAR"
    assert reg.mesh.eid2gids()[22] == {10, 12}
    assert reg.mesh.eid2gids(keep_order=True)[22] == [10, 12]
    eids_bbox = reg.mesh.eid2bbox()
    eid22_bbox = eids_bbox.loc[22].to_dict()
    assert eid22_bbox == {
        "X1_min": -37.5,
        "X2_min": 0.0,
        "X3_min": 10.0,
        "X1_max": -25.0,
        "X2_max": 0.0,
        "X3_max": 10.0,
    }



"""
test nastranio.writers.gmsh.Study

Read a gmsh mesh file
"""
from pathlib import Path

import pytest

from nastranio.readers.gmsh.study import Study
from nastranio.registry import Registry


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = Path(request.module.__file__)
    return filename.parent / "test_study"


def test_study(datadir, tmpdir):
    """if <meshfilename>.params.json exists next to <meshfilename>, parameters
    are retrived from it.
    This test check that "tapered.params.json" is correctly retrieved.
    """
    study = Study(datadir / "tapered.msh")
    study.build()
    bulk = study.to_nastran(Path(tmpdir) / "tapered.nas")
    assert bulk.exists()


def test_study_providing_params(datadir, tmpdir):
    """
    check Study import with explicit parameters file provided.
    """
    study = Study(datadir / "tapered.msh")
    study.load_user_params_from_file(datadir / "tapered.params.json")
    study.build()
    bulk = study.to_nastran(Path(tmpdir) / "tapered.nas")
    assert bulk.exists()
    # read newly created nastran bulk file
    _reg = Registry()
    _reg.from_bulkfile(bulk, nbprocs=8, progress=False)
    bulk = _reg.to_nastran(Path(tmpdir) / "tapered2.nas")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Low-Level Tests CTRIA3 card parser

    ref: NX Nastran 12 Quick Reference Guide 12-147 (p.1631)
"""

from pprint import pprint

import numpy as np
import pytest

from nastranio.cards import CTRIA3


@pytest.fixture
def parsed_bulk():
    cards = CTRIA3()
    # fmt: off
    cards_bulk = [
        """\
     $▔▔1▔▔▔╲╱▔▔2▔▔▔╲╱▔▔3▔▔▔╲╱▔▔4▔▔▔╲╱▔▔5▔▔▔╲╱▔▔6▔▔▔╲╱▔▔7▔▔▔╲╱▔▔8▔▔▔╲╱▔▔9▔▔▔╲╱▔▔10▔▔╲
     CTRIA3       111     203      31      74      75            0.98        +
     +                           1.77    20.4    2.09""",
    "CTRIA3         1       1       5      12      15     1.2",
    "CTRIA3         2       1       5      12      15       2",
    ]
    # fmt: on
    cards.parse(cards_bulk.pop(0))
    for card in cards_bulk:
        _ = CTRIA3()
        _.parse(card)
        cards.merge(_)
    return cards


def test_card_main(parsed_bulk):
    cards = parsed_bulk
    exp = {
        "EID": [111, 1, 2],
        "G1": [31, 5, 5],
        "G2": [74, 12, 12],
        "G3": [75, 15, 15],
        "MCID": [None, None, 2],
        "PID": [203, 1, 1],
        "T1": [1.77, None, None],
        "T2": [20.4, None, None],
        "T3": [2.09, None, None],
        "TFLAG": [None, None, None],
        "THETA": [0.0, 1.2, None],
        "THETA/MCID": [0.0, 1.2, 2],
        "ZOFFS": [0.98, None, None],
    }
    actual = cards.carddata["main"]
    pprint(actual)
    assert len(cards) == 3
    assert actual == exp


def test_export_data(parsed_bulk):
    cards = parsed_bulk
    # ========================================================================
    # test export_data() used for serialization
    # ========================================================================
    res = cards.export_data()
    exp = {
        "main": {
            "EID": [111, 1, 2],
            "G1": [31, 5, 5],
            "G2": [74, 12, 12],
            "G3": [75, 15, 15],
            "MCID": [None, None, 2],
            "PID": [203, 1, 1],
            "T1": [1.77, None, None],
            "T2": [20.4, None, None],
            "T3": [2.09, None, None],
            "TFLAG": [None, None, None],
            "THETA": [0.0, 1.2, None],
            "THETA/MCID": [0.0, 1.2, 2],
            "ZOFFS": [0.98, None, None],
        },
        "card": "CTRIA3",
    }

    assert res == exp


def test_to_nastran(parsed_bulk):
    """test to_nastran() method"""
    cards = parsed_bulk
    # # ========================================================================
    # # to_nastran
    # # ========================================================================
    nas = "\n".join(cards.to_nastran(ruler=True))
    print(nas)
    exp = """\
$CTRIA3▕    EID▕    PID▕     G1▕     G2▕     G3▕THETA..▕  ZOFFS▕       ▕+      ▕
$+     ▕       ▕  TFLAG▕     T1▕     T2▕     T3▕       ▕       ▕       ▕       ▕
CTRIA3       111     203      31      74      75      0.     .98        +
+                           1.77    20.4    2.09
CTRIA3         1       1       5      12      15     1.2
CTRIA3         2       1       5      12      15       2"""
    assert nas == exp


def test_to_nastran_nodefault(parsed_bulk):
    """test to_nastran() method without expliciting default values"""
    cards = parsed_bulk
    # # ========================================================================
    # # to_nastran
    # # ========================================================================
    nas = "\n".join(cards.to_nastran(ruler=True, with_default=False))
    print(nas)
    exp = """\
$CTRIA3▕    EID▕    PID▕     G1▕     G2▕     G3▕THETA..▕  ZOFFS▕       ▕+      ▕
$+     ▕       ▕  TFLAG▕     T1▕     T2▕     T3▕       ▕       ▕       ▕       ▕
CTRIA3       111     203      31      74      75             .98        +
+                           1.77    20.4    2.09
CTRIA3         1       1       5      12      15     1.2
CTRIA3         2       1       5      12      15       2"""
    assert nas == exp


def test_to_eid2gids(parsed_bulk):
    """test eid2gids() dictionnary"""
    cards = parsed_bulk
    eid2gids = cards.eid2gids()
    exp = {1: {12, 5, 15}, 2: {12, 5, 15}, 111: {74, 75, 31}}
    assert eid2gids == exp
    eid2gids = cards.eid2gids(keep_order=True)
    pprint(eid2gids)
    exp = {1: [5, 12, 15], 2: [5, 12, 15], 111: [31, 74, 75]}
    assert eid2gids == exp


def test_grinames(parsed_bulk):
    cards = parsed_bulk
    """ test gids_header() list
    """
    gridnames = cards.gids_header
    assert gridnames == ["G1", "G2", "G3"]


def test_thk(parsed_bulk):
    cards = parsed_bulk
    thks = cards.thk
    import pprint

    pprint.pprint(thks)  # TODO: remove me!
    np.testing.assert_allclose(thks["data"], np.array([8.08666667, np.nan, np.nan]))
    np.testing.assert_array_equal(thks["index"], np.array([111, 1, 2]))
    assert thks["name"] == "thk"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests nastranio.Registry"""

import os
from distutils import dir_util
from pprint import pprint as pp

import numpy as np
import pandas as pd
import pytest

from nastranio.cards import *
from nastranio.mesh_api import Mesh
from nastranio.registry import Registry

from .test_utils import df_from_csvstr


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = request.module.__file__
    basename, _ = os.path.splitext(filename)
    test_dir, model = basename.split("__")

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, str(tmpdir))

    return tmpdir


@pytest.fixture
def filled_reg(datadir):
    bulkpath = os.path.join(datadir, "test_panels.dat")
    _reg = Registry()
    _reg.from_bulkfile(bulkpath, nbprocs="auto", progress=False)
    return _reg


def test_00(filled_reg):
    reg = filled_reg
    pp(list(reg.container.keys()))
    exp = set(["exec", "params", "comments", "bulk", "cases", "meta", "summary"])
    assert set(reg.container.keys()) == exp


# def test_json_import_export(filled_reg):
#     reg = filled_reg
#     txt = reg.to_json()
#     reg2 = Registry()
#     reg2.from_json(txt)
#     assert reg == reg2


def test_msgpack_import_export(filled_reg):
    reg = filled_reg
    data = reg.to_msgpack()
    reg2 = Registry()
    reg2.from_msgpack(data)
    assert reg == reg2


def test_msgpack_serialize_unserialize(filled_reg):
    reg = filled_reg
    fname = reg.to_file(fmt="pack")
    reg2 = Registry()
    reg2.from_file(fname)
    assert reg == reg2


def test_json_serialize_unserialize(filled_reg):
    reg = filled_reg
    fname = reg.to_file(fmt="json")
    reg2 = Registry()
    reg2.from_file(fname)
    assert reg == reg2


def test_subset(filled_reg):
    reg = filled_reg
    # all_cq4_eids = set(reg.CQUAD4.array['EID'])
    # exp = {1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 24, 25}
    # assert exp == all_cq4_eids
    cq4_sub = reg.CQUAD4.subset(eids=(1, 4, 25))
    assert isinstance(cq4_sub, CQUAD4)
    assert set(cq4_sub.array["EID"]) == {1, 4, 25}


@pytest.mark.xfail(reason="test is not up-to-date")
def test_meetic(filled_reg):
    reg = filled_reg
    # standard panel specs
    panels_specs = {
        "samepid": True,
        "pcard1": "PCOMP",
        "pcard2": "PCOMP",
        "anglemax": 30,
        "min_paths": 2,
    }
    m = reg.mesh.meetic(**panels_specs)
    print(m.to_csv())
    exp = """\
gid,pathid,eid1,eid2,card1,dim1,pid1,card2,dim2,pid2,same_card,same_pid,same_dim,pcard1,pcard2,same_pcard,angle
2,0,1,2,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
3,0,2,3,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
4,0,3,4,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
6,1,1,23,CQUAD4,2d,6,CTRIA3,2d,6,False,True,True,PCOMP,PCOMP,True,0.0
6,2,5,23,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
7,0,1,2,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
7,3,1,23,CQUAD4,2d,6,CTRIA3,2d,6,False,True,True,PCOMP,PCOMP,True,0.0
7,4,2,6,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
7,7,6,22,CQUAD4,2d,6,CTRIA3,2d,6,False,True,True,PCOMP,PCOMP,True,0.0
7,9,22,23,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
8,0,2,3,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
8,1,2,6,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
8,5,3,7,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
8,7,6,7,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
9,0,3,4,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
9,1,3,7,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
9,5,4,8,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
9,7,7,8,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
10,0,4,8,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
11,1,5,21,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
12,3,6,22,CQUAD4,2d,6,CTRIA3,2d,6,False,True,True,PCOMP,PCOMP,True,0.0
12,5,9,10,CQUAD4,2d,7,CQUAD4,2d,7,True,True,True,PCOMP,PCOMP,True,0.0
12,12,21,22,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
13,0,6,7,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
13,5,7,11,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
14,0,7,8,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
14,1,7,11,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
14,5,8,12,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
14,7,11,12,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
15,0,8,12,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
17,0,9,10,CQUAD4,2d,7,CQUAD4,2d,7,True,True,True,PCOMP,PCOMP,True,0.0
17,5,13,14,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
18,5,14,15,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
21,0,13,17,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
22,0,13,14,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
22,1,13,17,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
22,4,14,18,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
22,5,17,18,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
23,0,14,15,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
23,1,14,18,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
23,4,15,19,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
23,5,18,19,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
24,0,15,16,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
24,1,15,19,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
24,4,16,20,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
24,5,19,20,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
25,0,16,20,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
27,0,17,18,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
28,0,18,19,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
29,0,19,20,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
31,0,5,21,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
31,2,5,23,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
31,3,21,22,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
31,5,22,23,CTRIA3,2d,6,CTRIA3,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
32,0,11,12,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
32,5,15,16,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
34,0,24,25,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
35,0,24,25,CQUAD4,2d,6,CQUAD4,2d,6,True,True,True,PCOMP,PCOMP,True,0.0
"""
    expected = df_from_csvstr(exp, index_col=[0, 1])
    m = m[[c for c in expected.columns]]
    expected.columns = m.columns
    assert len(m) == len(expected)
    errors = []
    # check by row...
    for i, row_m in m.iterrows():
        try:
            row_exp = expected.loc[i]
        except TypeError as exc:
            errors.append(i)
            continue
        try:
            assert row_exp.to_dict() == row_m.to_dict()
        except AssertionError:
            print(f"row #{i}")
            print(row_m)
            print("----")
            print(row_exp)
            raise
    if errors:
        print("errors: %s" % errors)
        assert 1 == 0


def test_ordered_eid2gids(filled_reg):

    reg = filled_reg
    e2g = reg.mesh.eid2gids(keep_order=True, eids=(1, 2, 23, 28))
    pp(e2g)
    exp = {1: [1, 6, 7, 2], 2: [2, 7, 8, 3], 23: [7, 6, 31], 28: [39, 10]}

    assert e2g == exp


def test_eid2gids(filled_reg):
    reg = filled_reg
    e2g = reg.mesh.eid2gids(eids=(1, 2, 23, 28))
    pp(e2g)
    exp = {1: {1, 6, 7, 2}, 2: {2, 7, 8, 3}, 23: {7, 6, 31}, 28: {39, 10}}
    assert e2g == exp


def test_to_vtk(filled_reg):
    reg = filled_reg
    _c = reg.mesh.eid2gids(keep_order=True)
    grid, gids_vtk2nas, eids_vtk2nas = reg.mesh.to_vtk(eids=(1, 2, 23, 28))
    # check if eid2gids still work
    _c2 = reg.mesh.eid2gids(keep_order=True)
    assert _c == _c2
    grid.save("panels.vtu")
    # try to load it
    import pyvista as pv

    g = pv.UnstructuredGrid("panels.vtu")
    os.remove("panels.vtu")
    pp(gids_vtk2nas)
    expected = {0: 1, 1: 2, 2: 3, 3: 6, 4: 7, 5: 8, 6: 10, 7: 31, 8: 39}
    assert gids_vtk2nas == expected


def test_normals(filled_reg):
    reg = filled_reg
    # mesh = Mesh()
    # mesh.set_registry(reg)
    normals = reg.mesh.normals(digits=3, eids=(1, 13, 28, 29))
    print(normals.to_csv())
    expected = """\
,X,Y,Z
1,-0.0,0.0,1.0
13,-0.5,0.0,0.866
28,0.0,0.0,-1.0
29,0.0,0.0,1.0"""

    expected = df_from_csvstr(expected)
    assert np.alltrue(normals == expected)


def test_area(filled_reg):
    reg = filled_reg
    areas = reg.mesh.area(eids=(1, 28, 13, 29))
    print(areas)
    assert np.allclose(
        areas["data"], np.array([0.75, np.nan, 1.0, np.nan]), equal_nan=True
    )


def test_thk(filled_reg):
    # Element 24 - LAMINATE PLATE    ( Quad, 4-noded )
    #  Property 6         Color 124    Layer 1         AttachTo 0
    # Element 27 - PLATE    ( Quad, 4-noded )
    #  Property 4         Color 124    Layer 1         AttachTo 0
    #  Plate Thickness     0.5           0.5           0.4           0.3
    # Element 35 - PLATE    ( Tri,  3-noded )
    #  Property 4         Color 124    Layer 1         AttachTo 0
    #  Plate Thickness     0.4           0.4           0.3
    # Element 36 - PLATE    ( Quad, 4-noded )
    #  Property 2         Color 124    Layer 1         AttachTo 0
    # Element 37 - PLATE    ( Tri,  3-noded )
    #  Property 2         Color 124    Layer 1         AttachTo 0
    # Element 33 - SPRING/DAMPER    ( Line, 2-noded )
    reg = filled_reg
    thks = reg.mesh.thk(eids=(24, 27, 4, 35, 36, 37, 33), asdict=False)
    exp = pd.Series(
        {
            24: 0.5,
            27: 0.425,
            4: 0.5,
            35: 0.3666666666666667,
            36: 0.5,
            37: 0.5,
            33: np.nan,
        }
    )
    np.testing.assert_allclose(exp.values, thks.values)
    np.testing.assert_array_equal(exp.index, thks.index)


def test_lengths(filled_reg):
    reg = filled_reg
    all_lengths = reg.mesh.length()
    # {
    #     'index': array([28, 29, 30, 31, 100, 200]),
    #     'data': array([0.5, 0.5, 0.5, 0.5, 0.5, 0.5]),
    #     'name': 'length',
    # }
    np.alltrue(all_lengths["index"] == np.array([28, 29, 30, 31, 100, 200]))
    np.allclose(all_lengths["data"], np.array([0.5, 0.5, 0.5, 0.5, 0.5, 0.5]))
    assert all_lengths["name"] == "length"
    # ------------------------------------------------------------------------
    # lengths by EIDS
    all_lengths = reg.mesh.length(eids=(100, 28))
    # {
    #     'index': array([28, 29, 30, 31, 100, 200]),
    #     'data': array([0.5, 0.5, 0.5, 0.5, 0.5, 0.5]),
    #     'name': 'length',
    # }
    np.alltrue(all_lengths["index"] == np.array([100, 28]))
    np.allclose(all_lengths["data"], np.array([0.5, 0.5]))
    assert all_lengths["name"] == "length"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Low-Level Tests RBE2 card parser

    ref: NX Nastran 12 Quick Reference Guide 17-45 (p.2519)
"""

from io import StringIO
from pprint import pprint

import pytest

from nastranio.cards import RBE2
from nastranio.registry import Registry


@pytest.fixture
def parsed_bulk():
    txt = """
BEGIN BULK
$▔▔1▔▔▔╲╱▔▔2▔▔▔╲╱▔▔3▔▔▔╲╱▔▔4▔▔▔╲╱▔▔5▔▔▔╲╱▔▔6▔▔▔╲╱▔▔7▔▔▔╲╱▔▔8▔▔▔╲╱▔▔9▔▔▔╲╱▔▔10▔▔╲
RBE2           9       8      12      10      12      14      15      16+
+             20
RBE2       12154    5124  123456    6170      2.
 """
    fh = StringIO()
    fh.write(txt)
    fh.seek(0)
    reg = Registry()
    reg.from_bulkfile(fh, nbprocs=1, progress=False)
    bulk = reg.container["bulk"]["RBE2"]
    return bulk


def test_card_main(parsed_bulk):
    cards = parsed_bulk
    exp = {
        "ALPHA": [0.0, 2.0],
        "CM": [12, 123456],
        "EID": [9, 12154],
        "GN": [8, 5124],
        "rbe2_gidsetID": [0, 1],
    }
    actual = cards.carddata["main"]
    assert actual == exp


def test_export_data(parsed_bulk):
    cards = parsed_bulk
    res = cards.export_data()
    exp = {
        "main": {
            "ALPHA": [0.0, 2.0],
            "CM": [12, 123456],
            "EID": [9, 12154],
            "GN": [8, 5124],
            "rbe2_gidsetID": [0, 1],
        },
        "card": "RBE2",
        "rbe2_gidset": [
            [{"GM": 10}, {"GM": 12}, {"GM": 14}, {"GM": 15}, {"GM": 16}, {"GM": 20}],
            [{"GM": 6170}],
        ],
    }

    assert res == exp


def test_to_nastran(parsed_bulk):
    """test to_nastran() method"""
    cards = parsed_bulk
    nas = "\n".join(cards.to_nastran(ruler=True))
    expected = """\
$RBE2  ▕    EID▕     GN▕     CM▕    GM1▕    GM2▕    GM3▕    GM4▕    GM5▕+      ▕
$+     ▕    GM6▕    GM7▕    GM8▕ -etc.-▕  ALPHA▕       ▕       ▕       ▕       ▕
RBE2           9       8      12      10      12      14      15      16+
+             20      0.
RBE2       12154    5124  123456    6170      2."""

    print(nas)
    assert nas == expected


def test_to_nastran_nodefault(parsed_bulk):
    """test to_nastran() method without expliciting default values"""
    cards = parsed_bulk
    nas = "\n".join(cards.to_nastran(ruler=True, with_default=False))
    expected = """\
$RBE2  ▕    EID▕     GN▕     CM▕    GM1▕    GM2▕    GM3▕    GM4▕    GM5▕+      ▕
$+     ▕    GM6▕    GM7▕    GM8▕ -etc.-▕  ALPHA▕       ▕       ▕       ▕       ▕
RBE2           9       8      12      10      12      14      15      16+
+             20
RBE2       12154    5124  123456    6170      2."""

    print(nas)
    assert nas == expected


def test_eid2gids(parsed_bulk):
    """test eid2gids() dictionnary"""
    cards = parsed_bulk
    eid2gids = cards.eid2gids()
    exp = {9: {16, 20, 8, 10, 12, 14, 15}, 12154: {6170, 5124}}
    assert eid2gids == exp
    eid2gids = cards.eid2gids(keep_order=True)
    pprint(eid2gids)
    exp = {9: [8, 10, 12, 14, 15, 16, 20], 12154: [5124, 6170]}
    assert eid2gids == exp


def test_grinames(parsed_bulk):
    cards = parsed_bulk
    """ test gids_header() list
    """
    gridnames = cards.gids_header
    assert gridnames == ["GN"]



""" collection of tools for tests """

from io import StringIO

import pandas as pd


def df_from_csvstr(txt, index_col=0):
    """create and return a pandas dataframe from passed string

    >>> txt = '''gid,X,Y,Z
    ...          1,374.793,-47.7918,14.6654
    ...          2,374.793,-46.965,14.6654'''
    >>> df_from_csvstr(txt)
               X        Y        Z
    gid
    1    374.793 -47.7918  14.6654
    2    374.793 -46.9650  14.6654
    """
    exp = StringIO(txt)
    df = pd.read_csv(exp, index_col=index_col)
    return df


if __name__ == "__main__":
    import doctest

    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Low-Level Tests CBUSH card parser

    ref: NX Nastran 12 Quick Reference Guide 11-39 (p.1399)
"""

from pprint import pprint

import pytest

from nastranio.cards import CBUSH


@pytest.fixture
def parsed_bulk():
    cards = CBUSH()
    # fmt: off
    cards_bulk = [
        """\
     $▔▔1▔▔▔╲╱▔▔2▔▔▔╲╱▔▔3▔▔▔╲╱▔▔4▔▔▔╲╱▔▔5▔▔▔╲╱▔▔6▔▔▔╲╱▔▔7▔▔▔╲╱▔▔8▔▔▔╲╱▔▔9▔▔▔╲╱▔▔10▔▔╲
     CBUSH         39       6       1     100      75""",
        """\
     $ GB not specified
     CBUSH         40       6       1                                       0""",
        """\
     $ Coincident Grid Points (usually used in VIP FEM)
     CBUSH         41       6       1     100                               6""",
        """\
     $ NONCOINCIDENT GRID POINTS WITH FIELDS 6 THROUGH 9 BLANK AND
     $ A SPRING-DAMPER OFFSET.
     CBUSH         42       6       1     600                                +
     +           0.25      10      0.     10.     10.""",
    ]
    # fmt: on
    cards.parse(cards_bulk.pop(0))
    for card in cards_bulk:
        _ = CBUSH()
        _.parse(card)
        cards.merge(_)
    return cards


def test_to_nastran(parsed_bulk):
    """test to_nastran() method"""
    cards = parsed_bulk
    # # ========================================================================
    # # to_nastran
    # # ========================================================================
    nas = "\n".join(cards.to_nastran(ruler=True))
    print(nas)
    exp = """\
$CBUSH ▕    EID▕    PID▕     GA▕     GB▕  GO/X1▕     X2▕     X3▕    CID▕+      ▕
$+     ▕      S▕   OCID▕     S1▕     S2▕     S3▕       ▕       ▕       ▕       ▕
CBUSH         39       6       1     100      75
CBUSH         40       6       1                                       0
CBUSH         41       6       1     100                               6
CBUSH         42       6       1     600                                +
+            .25      10      0.     10.     10."""

    assert nas == exp


def test_export_data(parsed_bulk):
    cards = parsed_bulk
    # ========================================================================
    # test export_data() used for serialization
    # ========================================================================
    res = cards.export_data()
    pprint(res)

    exp = {
        "card": "CBUSH",
        "main": {
            "CID": [None, 0, 6, None],
            "EID": [39, 40, 41, 42],
            "GA": [1, 1, 1, 1],
            "GB": [100, None, 100, 600],
            "GO": [75, None, None, None],
            "GO/X1": [75, None, None, None],
            "OCID": [None, None, None, 10],
            "PID": [6, 6, 6, 6],
            "S": [None, None, None, 0.25],
            "S1": [None, None, None, 0.0],
            "S2": [None, None, None, 10.0],
            "S3": [None, None, None, 10.0],
            "X1": [None, None, None, None],
            "X2": [None, None, None, None],
            "X3": [None, None, None, None],
        },
    }

    assert res == exp


def test_to_eid2gids(parsed_bulk):
    """test eid2gids() dictionnary"""
    cards = parsed_bulk
    eid2gids = cards.eid2gids()
    pprint(eid2gids)
    exp = {39: {1, 100}, 40: {1, None}, 41: {1, 100}, 42: {600, 1}}
    assert eid2gids == exp
    eid2gids = cards.eid2gids(keep_order=True)
    pprint(eid2gids)
    exp = {39: [1, 100], 40: [1, None], 41: [1, 100], 42: [1, 600]}
    assert eid2gids == exp


def test_grinames(parsed_bulk):
    cards = parsed_bulk
    """ test gids_header() list
    """
    gridnames = cards.gids_header
    assert gridnames == ["GA", "GB"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Low-Level Tests CQUAD4 card parser

    ref: NX Nastran 12 Quick Reference Guide 12-80 (p.1564)
"""

from pprint import pprint

import numpy as np
import pytest

from nastranio.cards import CQUAD4


@pytest.fixture
def parsed_bulk():
    cards = CQUAD4()
    # fmt: off
    cards_bulk = [
        """\
     $▔▔1▔▔▔╲╱▔▔2▔▔▔╲╱▔▔3▔▔▔╲╱▔▔4▔▔▔╲╱▔▔5▔▔▔╲╱▔▔6▔▔▔╲╱▔▔7▔▔▔╲╱▔▔8▔▔▔╲╱▔▔9▔▔▔╲╱▔▔10▔▔╲
     CQUAD4       111     203      31      74      75      32     2.6     0.3+
     +                      1    1.77    2.04    2.09    1.80""",
    "CQUAD4         1       1       5      12      15      13",
    "CQUAD4         2       1       5      12      15      13       2",
    ]
    # fmt: on
    cards.parse(cards_bulk.pop(0))
    for card in cards_bulk:
        _ = CQUAD4()
        _.parse(card)
        cards.merge(_)
    return cards


def test_card_main(parsed_bulk):
    cards = parsed_bulk
    exp = {
        "EID": [111, 1, 2],
        "G1": [31, 5, 5],
        "G2": [74, 12, 12],
        "G3": [75, 15, 15],
        "G4": [32, 13, 13],
        "MCID": [None, None, 2],
        "PID": [203, 1, 1],
        "T1": [1.77, None, None],
        "T2": [2.04, None, None],
        "T3": [2.09, None, None],
        "T4": [1.8, None, None],
        "TFLAG": [1, None, None],
        "THETA": [2.6, 0.0, None],
        "THETA/MCID": [2.6, 0.0, 2],
        "ZOFFS": [0.3, None, None],
    }
    actual = cards.carddata["main"]
    assert len(cards) == 3
    assert actual == exp


def test_export_data(parsed_bulk):
    cards = parsed_bulk
    # ========================================================================
    # test export_data() used for serialization
    # ========================================================================
    res = cards.export_data()
    pprint(res)
    exp = {
        "main": {
            "EID": [111, 1, 2],
            "G1": [31, 5, 5],
            "G2": [74, 12, 12],
            "G3": [75, 15, 15],
            "G4": [32, 13, 13],
            "MCID": [None, None, 2],
            "PID": [203, 1, 1],
            "T1": [1.77, None, None],
            "T2": [2.04, None, None],
            "T3": [2.09, None, None],
            "T4": [1.8, None, None],
            "TFLAG": [1, None, None],
            "THETA": [2.6, 0.0, None],
            "THETA/MCID": [2.6, 0.0, 2],
            "ZOFFS": [0.3, None, None],
        },
        "card": "CQUAD4",
    }

    assert res == exp


def test_to_nastran(parsed_bulk):
    """test to_nastran() method"""
    cards = parsed_bulk
    # # ========================================================================
    # # to_nastran
    # # ========================================================================
    nas = "\n".join(cards.to_nastran(ruler=True))
    print(nas)
    exp = """\
$CQUAD4▕    EID▕    PID▕     G1▕     G2▕     G3▕     G4▕THETA..▕  ZOFFS▕+      ▕
$+     ▕       ▕  TFLAG▕     T1▕     T2▕     T3▕     T4▕       ▕       ▕       ▕
CQUAD4       111     203      31      74      75      32     2.6      .3+
+                      1    1.77    2.04    2.09     1.8
CQUAD4         1       1       5      12      15      13      0.
CQUAD4         2       1       5      12      15      13       2"""

    assert nas == exp


def test_to_nastran_nodefault(parsed_bulk):
    """test to_nastran() method without expliciting default values"""
    cards = parsed_bulk
    # # ========================================================================
    # # to_nastran
    # # ========================================================================
    nas = "\n".join(cards.to_nastran(ruler=True, with_default=False))
    print(nas)
    exp = """\
$CQUAD4▕    EID▕    PID▕     G1▕     G2▕     G3▕     G4▕THETA..▕  ZOFFS▕+      ▕
$+     ▕       ▕  TFLAG▕     T1▕     T2▕     T3▕     T4▕       ▕       ▕       ▕
CQUAD4       111     203      31      74      75      32     2.6      .3+
+                      1    1.77    2.04    2.09     1.8
CQUAD4         1       1       5      12      15      13
CQUAD4         2       1       5      12      15      13       2"""

    assert nas == exp


def test_to_eid2gids(parsed_bulk):
    """test eid2gids() dictionnary"""
    cards = parsed_bulk
    eid2gids = cards.eid2gids()
    exp = {1: {13, 12, 5, 15}, 2: {13, 12, 5, 15}, 111: {32, 74, 75, 31}}
    assert eid2gids == exp
    eid2gids = cards.eid2gids(keep_order=True)
    exp = {1: [5, 12, 15, 13], 2: [5, 12, 15, 13], 111: [31, 74, 75, 32]}
    assert eid2gids == exp


def test_grinames(parsed_bulk):
    cards = parsed_bulk
    """ test gids_header() list
    """
    gridnames = cards.gids_header
    assert gridnames == ["G1", "G2", "G3", "G4"]


# def test_json(parsed_bulk):
#     cards = parsed_bulk
#     cards2 = CQUAD4()
#     cards2.from_json(cards.to_json())
#     assert cards2 == cards


def test_msgpack(parsed_bulk):
    cards = parsed_bulk
    cards2 = CQUAD4()
    cards2.from_msgpack(cards.to_msgpack())
    assert cards2 == cards


def test_thk(parsed_bulk):
    cards = parsed_bulk
    thks = cards.thk
    import pprint

    pprint.pprint(thks)  # TODO: remove me!
    np.testing.assert_array_equal(thks["data"], np.array([1.925, np.nan, np.nan]))
    np.testing.assert_array_equal(thks["index"], np.array([111, 1, 2]))
    assert thks["name"] == "thk"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Low-Level Tests GRID card parser

    ref: NX Nastran 12 Quick Reference Guide 14-60 (p.1926)
"""

from io import StringIO
from pprint import pprint

import numpy as np
import pandas as pd
import pytest
from pandas.util.testing import assert_frame_equal

from nastranio.cards import GRID

from .test_utils import df_from_csvstr


@pytest.fixture
def parsed_bulk():
    cards = GRID()
    # fmt: off
    cards_bulk = [
        """\
     $▔▔1▔▔▔╲╱▔▔2▔▔▔╲╱▔▔3▔▔▔╲╱▔▔4▔▔▔╲╱▔▔5▔▔▔╲╱▔▔6▔▔▔╲╱▔▔7▔▔▔╲╱▔▔8▔▔▔╲╱▔▔9▔▔▔╲╱▔▔10▔▔╲
     GRID           1       0 374.793-47.7918 14.6654       0""",
    "GRID           2       0 374.793 -46.965 14.6654       0     123       2"
    ]
    # fmt: on
    cards.parse(cards_bulk.pop(0))
    for card in cards_bulk:
        _ = GRID()
        _.parse(card)
        cards.merge(_)
    return cards


def test_card_main(parsed_bulk):
    cards = parsed_bulk
    exp = {
        "ID": [1, 2],
        "CP": [0, 0],
        "X1": [374.793, 374.793],
        "X2": [-47.7918, -46.965],
        "X3": [14.6654, 14.6654],
        "CD": [0, 0],
        "PS": [None, 123],
        "SEID": [None, 2],
    }
    assert cards.carddata["main"] == exp


def test_export_data(parsed_bulk):
    cards = parsed_bulk
    # ========================================================================
    # test export_data() used for serialization
    # ========================================================================
    res = cards.export_data()
    exp = {
        "main": {
            "ID": [1, 2],
            "CP": [0, 0],
            "X1": [374.793, 374.793],
            "X2": [-47.7918, -46.965],
            "X3": [14.6654, 14.6654],
            "CD": [0, 0],
            "PS": [None, 123],
            "SEID": [None, 2],
        },
        "card": "GRID",
    }
    assert res == exp


def test_to_nastran(parsed_bulk):
    """test to_nastran() method"""
    cards = parsed_bulk
    # test that nastran's output match previous input
    nas = "\n".join(cards.to_nastran(ruler=False))
    expected = """\
$GRID  ▕     ID▕     CP▕     X1▕     X2▕     X3▕     CD▕     PS▕   SEID▕       ▕
GRID           1       0 374.793-47.7918 14.6654       0
GRID           2       0 374.793 -46.965 14.6654       0     123       2"""
    nas = "\n".join(cards.to_nastran(ruler=True))
    assert nas == expected


def test_coords(parsed_bulk):
    cards = parsed_bulk
    gids, coords, csys = cards.coords()
    pprint(gids)
    pprint(coords)
    assert np.alltrue(gids == np.array([1, 2]))
    assert np.alltrue(
        coords == np.array([[374.793, -47.7918, 14.6654], [374.793, -46.965, 14.6654]])
    )


def test_coords_dataframe(parsed_bulk):
    cards = parsed_bulk
    coords = cards.coords(asdf=True)
    exp = "gid,csys,X,Y,Z\n1,0,374.793,-47.7918,14.6654\n2,0,374.793,-46.965,14.6654\n"
    expected = df_from_csvstr(exp)
    pprint(coords)
    pprint(expected)
    assert isinstance(coords, pd.DataFrame)
    assert_frame_equal(coords, expected)


def test_selected_coords(parsed_bulk):
    cards = parsed_bulk
    gids, coords, csys = cards.coords(gids=[-1, 2])
    pprint(gids)
    pprint(coords)
    assert np.alltrue(gids == np.array([2]))
    assert np.alltrue(coords == np.array([[374.793, -46.965, 14.6654]]))


# def test_json(parsed_bulk):
#     cards = parsed_bulk
#     cards2 = GRID()
#     cards2.from_json(cards.to_json())
#     assert cards2 == cards


def test_msgpack(parsed_bulk):
    cards = parsed_bulk
    arr = cards.array
    cards2 = GRID()
    cards2.from_msgpack(cards.to_msgpack())
    assert cards2 == cards



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Low-Level Tests CBAR card parser

    ref: NX Nastran 12 Quick Reference Guide 11-20 (p.1380)
"""

from pprint import pprint

import pytest

from nastranio.cards import CBAR


@pytest.fixture
def parsed_bulk():
    cards = CBAR()
    # fmt: off
    cards_bulk = [
        """\
     $▔▔1▔▔▔╲╱▔▔2▔▔▔╲╱▔▔3▔▔▔╲╱▔▔4▔▔▔╲╱▔▔5▔▔▔╲╱▔▔6▔▔▔╲╱▔▔7▔▔▔╲╱▔▔8▔▔▔╲╱▔▔9▔▔▔╲╱▔▔10▔▔╲
     CBAR           2      39       7       3     0.6     18.     26.        +
     +                    513""",
        """\
     $ ALTERNATE FORMAT
     CBAR           3      39       7       6     10                         +
     +                    513"""
    ]
    # fmt: on
    cards.parse(cards_bulk.pop(0))
    for card in cards_bulk:
        _ = CBAR()
        _.parse(card)
        cards.merge(_)
    return cards


def test_export_data(parsed_bulk):
    cards = parsed_bulk
    # ========================================================================
    # test export_data() used for serialization
    # ========================================================================
    res = cards.export_data()
    pprint(res)

    exp = {
        "main": {
            "EID": [2, 3],
            "GO": [None, 10],
            "GO/X1": [0.6, 10],
            "GA": [7, 7],
            "GB": [3, 6],
            "PA": [None, None],
            "PB": [513, 513],
            "PID": [39, 39],
            "W1A": [None, None],
            "W1B": [None, None],
            "W2A": [None, None],
            "W2B": [None, None],
            "W3B": [None, None],
            "W3A": [None, None],
            "X1": [0.6, None],
            "X2": [18.0, None],
            "X3": [26.0, None],
        },
        "card": "CBAR",
    }

    assert res == exp


def test_to_nastran(parsed_bulk):
    """test to_nastran() method"""
    cards = parsed_bulk
    # # ========================================================================
    # # to_nastran
    # # ========================================================================
    nas = "\n".join(cards.to_nastran(ruler=True))
    print(nas)

    exp = """\
$CBAR  ▕    EID▕    PID▕     GA▕     GB▕  GO/X1▕     X2▕     X3▕       ▕+      ▕
$+     ▕     PA▕     PB▕    W1A▕    W2A▕    W3A▕    W1B▕    W2B▕    W3B▕       ▕
CBAR           2      39       7       3      .6     18.     26.        +
+                    513
CBAR           3      39       7       6      10                        +
+                    513"""

    assert nas == exp


def test_to_eid2gids(parsed_bulk):
    """test eid2gids() dictionnary"""
    cards = parsed_bulk
    eid2gids = cards.eid2gids()
    exp = {2: {3, 7}, 3: {6, 7}}
    assert eid2gids == exp
    eid2gids = cards.eid2gids(keep_order=True)
    exp = {2: [7, 3], 3: [7, 6]}
    assert eid2gids == exp


def test_grinames(parsed_bulk):
    cards = parsed_bulk
    """ test gids_header() list
    """
    gridnames = cards.gids_header
    assert gridnames == ["GA", "GB"]



"""Modificatin Module testing"""

import os
from distutils import dir_util
from pprint import pprint as pp

import numpy as np
import pytest

from nastranio.cards import *
from nastranio.mesh_api import Mesh
from nastranio.registry import Registry

from .test_utils import df_from_csvstr


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = request.module.__file__
    basename, _ = os.path.splitext(filename)
    test_dir, model = basename.split("__")

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, str(tmpdir))

    return tmpdir


@pytest.fixture
def filled_reg(datadir):
    bulkpath = os.path.join(datadir, "stiffpanel.nas")
    _reg = Registry()
    _reg.from_bulkfile(bulkpath, nbprocs=8, progress=False)
    _reg.bind_mod_api(index_elems=("1d", "2d"))
    return _reg


def test_closest(filled_reg):
    reg = filled_reg
    closest_gids = reg.mesh.mod.rtrees.nearest_grids(
        coords=(-2, -1, 0), num_results=1, astype=list
    )
    assert closest_gids == [1]
    closest_eids = reg.mesh.mod.rtrees.nearest_elements(
        coords=(-2, -1, 0), num_results=1, astype=set
    )
    assert closest_eids == {"CBAR": {1, 28}, "CTRIA3": {106, 100}}


def test_move_node(filled_reg):
    reg = filled_reg
    # breakpoint()
    # move node#1 twice.
    reg.mesh.mod.node_move_by(1, vector=[-0.2, 0, 0.1])
    reg.mesh.mod.node_move_by(1, vector=[-0.2, 0, 0.1])
    # check new grid position
    xyz_fast = reg.container["bulk"]["GRID"].query_id_fast(
        1, columns=["X1", "X2", "X3"], asview=True
    )
    assert np.allclose(xyz_fast, [-2.4, -1, 0.2])
    xyz_slow = reg.container["bulk"]["GRID"].query_id(1, asview=float)[2:5]
    assert np.allclose(xyz_slow, [-2.4, -1, 0.2])
    xyz12 = reg.container["bulk"]["GRID"].query_id_fast(
        (1, 2), columns=["X1", "X2", "X3"], asview=True
    )
    assert np.allclose(xyz12, [[-2.4, -1.0, 0.2], [2.0, -1.0, 0.0]])

    xyz21 = reg.container["bulk"]["GRID"].query_id_fast(
        (2, 1), columns=["X2", "X1", "X3"], asview=True
    )
    assert np.allclose(xyz21, [[-1.0, 2.0, 0.0], [-1.0, -2.4, 0.2]])
    # check bboxes
    closest_gids = reg.mesh.mod.rtrees.nearest_grids(
        coords=(-2, -1, 0), num_results=1, astype=list
    )
    assert closest_gids == [28]
    closest_eids = reg.mesh.mod.rtrees.nearest_elements(
        coords=(-2, -1, 0), num_results=1, astype=set
    )
    assert closest_eids == {"CBAR": {1, 28}, "CTRIA3": {106, 100}}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests nastranio.Registry"""

import os
from distutils import dir_util
from pprint import pprint as pp

import numpy as np
import pytest

from nastranio.cards import *
from nastranio.mesh_api import Mesh
from nastranio.registry import Registry

from .test_utils import df_from_csvstr


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = request.module.__file__
    basename, _ = os.path.splitext(filename)
    test_dir, model = basename.split("__")

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, str(tmpdir))

    return tmpdir


@pytest.fixture
def filled_reg(datadir):
    bulkpath = os.path.join(datadir, "nightstand.dat")
    _reg = Registry()
    _reg.from_bulkfile(bulkpath, nbprocs=8, progress=False)
    return _reg


def test_coords_as_defined_by_gids(filled_reg):
    reg = filled_reg
    # mesh = Mesh()
    # mesh.set_registry(reg)
    # ------------------------------------------------------------------------
    # as numpy data
    #     ID    Def CS      X1           X2           X3
    # _______________________________________________________
    #       1       11   19.00522    -47.01603     15.80569
    #      13        0    375.331     -47.3115      14.6654
    gids, coords, csys = reg.mesh.coords(gids=(1, 13))
    expected_gids = np.array([1, 13])
    expected_coords = np.array(
        [[374.79303586, -47.79179422, 14.66539227], [375.331, -47.3115, 14.6654]]
    )
    expected_csys = np.array([0, 0])
    assert np.alltrue(gids == expected_gids)
    assert np.allclose(coords, expected_coords)
    assert np.alltrue(csys == expected_csys)
    # ------------------------------------------------------------------------
    # as pandas DataFrame
    coords = reg.mesh.coords(gids=(1, 13), asdf=True)
    expected_csv = """\
gid,csys,X,Y,Z
1,0,374.79303585921286,-47.79179421589026,14.665392265440204
13,0,375.331,-47.3115,14.6654
"""
    expected = df_from_csvstr(expected_csv)
    print(coords)
    print(expected)
    assert np.allclose(coords, expected)


def test_coords_by_gids(filled_reg):
    reg = filled_reg
    # mesh = Mesh()
    # mesh.set_registry(reg)
    # ------------------------------------------------------------------------
    # as numpy data
    #     ID    Def CS          X1           X2           X3
    # ___________________   Listing in Coordinate System    0   --------
    #       1       11        374.793     -47.7918      14.6654
    #      13        0        375.331     -47.3115      14.6654
    gids, coords, csys = reg.mesh.coords(gids=(1, 13), incsys=0)
    expected_gids = np.array([1, 13])
    expected_coords = np.array(
        [[374.793, -47.7918, 14.6654], [375.331, -47.3115, 14.6654]]
    )
    expected_csys = np.array([0, 0])
    print(gids)
    assert np.allclose(gids, expected_gids)
    print(csys)
    assert np.allclose(csys, expected_csys)
    print(coords)
    assert np.allclose(coords, expected_coords)
    # ------------------------------------------------------------------------
    # as pandas DataFrame
    coords = reg.mesh.coords(gids=(1, 13), asdf=True)
    expected_csv = """\
gid,csys,X,Y,Z
1,0,374.79303585921286,-47.79179421589026,14.665392265440204
13,0,375.331,-47.3115,14.6654"""
    expected = df_from_csvstr(expected_csv)
    print(coords)
    print(expected)
    assert np.allclose(coords, expected)
    # ------------------------------------------------------------------------
    # as pandas DataFrame in ginven CSYS
    coords = reg.mesh.coords(gids=(1, 13), asdf=True, incsys=0)
    expected_csv = """\
gid,csys,X,Y,Z
1,0,374.793,-47.7918,14.6654
13,0,375.331,-47.3115,14.6654"""
    expected = df_from_csvstr(expected_csv)
    print("GOT:")
    print("----")
    print(coords)
    print("EXPECTING:")
    print("----------")
    print(expected)
    assert np.allclose(coords, expected)


def test_coords_as_defined_by_eids(filled_reg):
    reg = filled_reg
    # ------------------------------------------------------------------------
    # searching by eids
    gids, coords, csys = reg.mesh.coords(eids=(1, 8))
    print("coords:", coords)
    print("gids:", gids)
    print("csys:", csys)
    expected_coords = np.array(
        [
            [374.79303586, -47.79179422, 14.66539227],
            [375.856, -47.402, 14.6654],
            [374.793, -47.3784, 14.6654],
            [375.686, -47.6245, 14.6654],
            [375.331, -47.3115, 14.6654],
            [375.216, -47.5571, 14.6654],
        ]
    )
    expected_gids = np.array([1, 5, 8, 12, 13, 15])
    expected_csys = np.array([0, 0, 0, 0, 0, 0])
    assert np.allclose(coords, expected_coords)
    assert np.alltrue(gids == expected_gids)
    assert np.alltrue(csys == expected_csys)
    # ------------------------------------------------------------------------
    # as pandas DataFrame
    coords = reg.mesh.coords(eids=(1, 8), asdf=True)
    expected_csv = """\
gid,csys,X,Y,Z
1,0,374.79303585921286,-47.79179421589026,14.665392265440204
5,0,375.856,-47.402,14.6654
8,0,374.793,-47.3784,14.6654
12,0,375.686,-47.6245,14.6654
13,0,375.331,-47.3115,14.6654
15,0,375.216,-47.5571,14.6654"""
    expected = df_from_csvstr(expected_csv)
    print(coords)
    print(coords.to_csv())
    print(expected)
    assert np.allclose(coords, expected)


def test_boundaries(filled_reg):
    boundaries = filled_reg.mesh.boundaries
    assert boundaries.T.to_dict() == {
        12508: {"dof": 123, "sid": 290, "source": "SPC1"},
        12509: {"dof": 123, "sid": 290, "source": "SPC1"},
        12510: {"dof": 123, "sid": 290, "source": "SPC1"},
        12511: {"dof": 123, "sid": 290, "source": "SPC1"},
        12516: {"dof": 123456, "sid": -1, "source": "GRID"},
        12517: {"dof": 123456, "sid": -1, "source": "GRID"},
    }


def test_loading(filled_reg):
    loads = filled_reg.mesh.load_combination
    assert loads is None


def test_to_gmsh(filled_reg):
    txt = filled_reg.mesh.to_gmsh()


def test_to_nastran(filled_reg):
    txt = filled_reg.to_nastran()



import os
from distutils import dir_util

import pytest

from nastranio.registry import Registry


@pytest.fixture
def datadir(tmpdir, request):
    """
    Fixture responsible for searching a folder with the same name of test
    module and, if available, moving all contents to a temporary directory so
    tests can use them freely.
    """
    filename = request.module.__file__
    basename, _ = os.path.splitext(filename)
    test_dir, model = basename.split("__")

    if os.path.isdir(test_dir):
        dir_util.copy_tree(test_dir, str(tmpdir))

    return tmpdir


@pytest.fixture
def filled_reg(datadir):
    bulkpath = os.path.join(datadir, "bug310.nas")
    _reg = Registry()
    _reg.from_bulkfile(bulkpath, nbprocs=8, progress=False)
    return _reg


def test_coords_as_defined_by_gids(datadir):
    bulkpath = os.path.join(datadir, "bug310.nas")
    reg1 = Registry()
    reg1.from_bulkfile(bulkpath, nbprocs=1, progress=False)
    elapsed = {1: reg1.container["meta"]["elapsed"]}
    for nbprocs in (2, 3, 4, 5):
        regi = Registry()
        regi.from_bulkfile(bulkpath, nbprocs=nbprocs, progress=False)
        diff = reg1.diff(regi)["values_changed"]
        assert "root['meta']['nbprocs']" in set(diff.keys())
        assert reg1 == regi
        elapsed[nbprocs] = regi.container["meta"]["elapsed"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import glob

"""The setup script."""

from setuptools import find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest>=3",
]

data_files = glob.glob("nastranio/writers/templates/*.tpl", recursive=True)

setup(
    author="Nicolas Cordier",
    author_email="nicolas.cordier@numeric-gmbh.ch",
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.8",
    ],
    description="Efficient Nastran Parser",
    install_requires=requirements,
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="nastranio",
    name="nastranio",
    packages=find_packages(include=["nastranio", "nastranio.*"]),
    data_files=[
        (
            "nastranio/writers/templates",
            data_files,
        )
    ],
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    version="0.16.0",
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "nastranio=nastranio.cli:main",
        ],
    },
)
